﻿using Hotline.Application.FlowEngine;
using Hotline.Application.Knowledge;
using Hotline.FlowEngine.Workflows;
using Hotline.KnowledgeBase;
using Hotline.KnowledgeBase.Notifies;
using Hotline.Permissions;
using Hotline.Repository.SqlSugar.Extensions;
using Hotline.Settings;
using Hotline.Share.Dtos;
using Hotline.Share.Dtos.FlowEngine;
using Hotline.Share.Dtos.Knowledge;
using Hotline.Share.Enums.KnowledgeBase;
using MapsterMapper;
using MediatR;
using Microsoft.AspNetCore.Mvc;
using Microsoft.IdentityModel.Tokens;
using SqlSugar;
using XF.Domain.Authentications;
using XF.Domain.Exceptions;

namespace Hotline.Api.Controllers
{
    /// <summary>
    /// 
    /// </summary>
    public class KnowledgeController : BaseController
    {
        private readonly IKnowledgeRepository _knowledgeRepository;
        private readonly IKnowledgeTempRepository _knowledgeTempRepository;
        private readonly ISessionContext _sessionContext;
        private readonly IKnowledgeDomainService _knowledgeDomainService;
        private readonly IMapper _mapper;
        private readonly IKnowApplication _knowApplication;
        private readonly IMediator _mediator;
        private readonly IWorkflowApplication _workflowApplication;
        private readonly IWorkflowDomainService _workflowDomainService;
        private readonly IKnowledgeWorkFlowRepository _knowledgeWorkFlowRepository;


        /// <summary>
        /// 
        /// </summary>
        /// <param name="knowledgeRepository"></param>
        /// <param name="knowledgeTempRepository"></param>
        /// <param name="sessionContext"></param>
        /// <param name="knowledgeDomainService"></param>
        /// <param name="mapper"></param>
        /// <param name="knowApplication"></param>
        /// <param name="mediator"></param>
        /// <param name="workflowApplication"></param>
        /// <param name="workflowDomainService"></param>
        /// <param name="knowledgeWorkFlowRepository"></param>
        public KnowledgeController(IKnowledgeRepository knowledgeRepository, IKnowledgeTempRepository knowledgeTempRepository,
           ISessionContext sessionContext, IKnowledgeDomainService knowledgeDomainService, IMapper mapper, IKnowApplication knowApplication,
           IMediator mediator, IWorkflowApplication workflowApplication, IWorkflowDomainService workflowDomainService, IKnowledgeWorkFlowRepository knowledgeWorkFlowRepository)
        {
            _knowledgeRepository = knowledgeRepository;
            _knowledgeTempRepository = knowledgeTempRepository;
            _sessionContext = sessionContext;
            _knowledgeDomainService = knowledgeDomainService;
            _mapper = mapper;
            _knowApplication = knowApplication;
            _mediator = mediator;
            _workflowApplication = workflowApplication;
            _workflowDomainService = workflowDomainService;
            _knowledgeWorkFlowRepository = knowledgeWorkFlowRepository;
        }
        #region 草稿箱
        /// <summary>
        /// 草稿箱-新增
        /// </summary>
        /// <param name="dto"></param>
        /// <returns></returns>
        [Permission(EPermission.KnowledgeDraftsAdd)]
        [HttpPost("draftsadd")]
        public async Task<string> KnowledgeDraftsAdd([FromBody] AddKnowledgeDto dto)
        {
            var kn = _mapper.Map<KnowledgeTemp>(dto);
            if (string.IsNullOrEmpty(dto.Code))
                kn.Code = Convert.ToInt64((DateTime.Now - new DateTime(1970, 1, 1, 0, 0, 0, 0)).TotalSeconds).ToString();

            return await _knowledgeTempRepository.AddAsync(kn, HttpContext.RequestAborted);
        }

        /// <summary>
        /// 草稿箱-修改
        /// </summary>
        /// <param name="dto"></param>
        /// <returns></returns>
        [Permission(EPermission.KnowledgeDraftsUpdate)]
        [HttpPut("draftsupdate")]
        public async Task KnowledgeDraftsUpdate([FromBody] UpdateKnowledgeDto dto)
        {
            var kn = _mapper.Map<KnowledgeTemp>(dto);
            await _knowledgeTempRepository.UpdateAsync(kn, HttpContext.RequestAborted);
        }

        /// <summary>
        /// 草稿箱-查询详情
        /// </summary>
        /// <param name="Id"></param>
        /// <returns></returns>
        [HttpGet("draftsinfo/{id}")]
        public async Task<KnowledgeTemp> KnowledgeDraftsInfo(string Id)
        {
            var temp = await _knowledgeTempRepository.GetAsync(Id, HttpContext.RequestAborted);
            if (temp is null)
                throw UserFriendlyException.SameMessage("知识草稿查询失败！");
            return temp;
        }

        /// <summary>
        /// 草稿箱删除
        /// </summary>
        /// <param name="Id"></param>
        /// <returns></returns>
        [Permission(EPermission.KnowledgeDraftsDelete)]
        [HttpDelete("draftsdelete")]
        public async Task KnowledgeDraftsDelete(string Id)
        {
            if (string.IsNullOrEmpty(Id))
                throw UserFriendlyException.SameMessage("Id不能为空！");
            await _knowledgeTempRepository.RemoveAsync(p => p.Id == Id, false, HttpContext.RequestAborted);
        }

        /// <summary>
        /// 我的草稿箱
        /// </summary>
        /// <param name="pagedDto"></param>
        /// <returns></returns>
        [Permission(EPermission.KnowledgeMyDraftsList)]
        [HttpPost("mydraftslist")]
        public async Task<PagedDto<KnowledgeDataDto>> MyDraftsList([FromBody] MyDraftsListPagedDto pagedDto)
        {
            var (total, items) = await _knowledgeTempRepository
             .Queryable()
             .Includes(it => it.User)
             .Includes(it => it.KnowledgeType)
             .Where(p => p.CreatorId == _sessionContext.RequiredUserId)
             .WhereIF(!string.IsNullOrEmpty(pagedDto.Keyword), d => d.Title.Contains(pagedDto.Keyword!))
             .WhereIF(!string.IsNullOrEmpty(pagedDto.KnowledgeTypeId), d => d.KnowledgeTypeId == pagedDto.KnowledgeTypeId)
             .WhereIF(pagedDto.StartTime != null, d => d.CreationTime >= pagedDto.StartTime)
             .WhereIF(pagedDto.EndTime != null, d => d.CreationTime <= pagedDto.EndTime)
             .OrderByDescending(p => p.CreationTime)
             .ToPagedListAsync(pagedDto.PageIndex, pagedDto.PageSize, HttpContext.RequestAborted);

            return new PagedDto<KnowledgeDataDto>(total, _mapper.Map<IReadOnlyList<KnowledgeDataDto>>(items));
        }
        #endregion

        #region 知识管理
        /// <summary>
        /// 知识库-知识修改-查询详情
        /// </summary>
        /// <param name="Id"></param>
        /// <returns></returns>
        [HttpGet("updateinfo/{Id}")]
        public async Task<Knowledge> KnowledgeUpdateInfo(string Id)
        {
            var know = await _knowledgeRepository.GetAsync(Id, HttpContext.RequestAborted);
            if (know is null)
                throw UserFriendlyException.SameMessage("知识查询失败！");
            return know;
        }

        /// <summary>
        /// 知识库-查询详情-增加浏览量
        /// </summary>
        /// <param name="Id"></param>
        /// <returns></returns>
        [HttpGet("info/{Id}")]
        public async Task<Knowledge> KnowledgeInfo(string Id)
        {
            var knowledge = await _knowledgeDomainService.KnowledgeInfo(Id, HttpContext.RequestAborted);
            _mediator.Publish(new GetKnowledgeInfoNotify(knowledge));
            return knowledge;
        }

        /// <summary>
        /// 知识库-新增
        /// </summary>
        /// <param name="dto"></param>
        /// <returns></returns>
        [Permission(EPermission.AddKnowledge)]
        [HttpPost("knowadd")]
        public async Task<string> KnowledgeAdd([FromBody] AddKnowledgeDto dto)
        {
            //知识处理逻辑
            var knowledgeId = await _knowledgeDomainService.KnowledgeAdd(dto, HttpContext.RequestAborted);
            //处理流程
            if (!string.IsNullOrEmpty(knowledgeId))
            {
                //调用流程创建
                var workFlowId = await _workflowApplication.StartWorkflowAsync(dto.StartWorkflowDto, HttpContext.RequestAborted);

                //知识审批主表
                await _knowledgeDomainService.AddWorkFlow(knowledgeId, workFlowId, EKnowledgeApplyType.Add, HttpContext.RequestAborted);

                //修改流程ID
                await _knowledgeDomainService.UpdateWorkFlowId(knowledgeId, workFlowId, HttpContext.RequestAborted);

            }
            return knowledgeId;
        }

        /// <summary>
        /// 知识库-修改
        /// </summary>
        /// <param name="dto"></param>
        /// <returns></returns>
        [Permission(EPermission.KnowledgeUpdate)]
        [HttpPut("knowupdate")]
        public async Task KnowledgeUpdate([FromBody] UpdateKnowledgeDto dto)
        {
            //知识更新流程业务
            var knowId = await _knowledgeDomainService.KnowledgeUpdate(dto, HttpContext.RequestAborted);
            if (!string.IsNullOrEmpty(knowId))
            {
                //调用流程创建
                var workFlowId = await _workflowApplication.StartWorkflowAsync(dto.StartWorkflowDto, HttpContext.RequestAborted);

                //知识审批主表
                await _knowledgeDomainService.AddWorkFlow(knowId, workFlowId, EKnowledgeApplyType.Update, HttpContext.RequestAborted);

                //修改流程ID
                await _knowledgeDomainService.UpdateWorkFlowId(knowId, workFlowId, HttpContext.RequestAborted);

            }
        }

        /// <summary>
        /// 知识库-删除
        /// </summary>
        /// <param name="Id"></param>
        /// <param name="workDto"></param>
        /// <returns></returns>
        [Permission(EPermission.KnowledgeDelete)]
        [HttpDelete("knowdelete")]
        public async Task KnowledgeDelete(string Id, [FromBody] StartWorkflowDto workDto)
        {
            var know = await _knowledgeRepository.GetAsync(Id, HttpContext.RequestAborted);
            if (know != null && know.IsDeleted == false)
            {
                //验证是否已经发起过知识删除流程
                var exists = _knowledgeWorkFlowRepository.GetAsync(p => p.KnowledgeId == know.Id && p.WorkflowModuleStatus == EKnowledgeApplyType.Delete
                && (p.WorkFlowApplyStatus != EKnowledgeWorkFlowStatus.Success || p.WorkFlowApplyStatus != EKnowledgeWorkFlowStatus.Failed));
                if (exists != null)
                    throw UserFriendlyException.SameMessage($"该知识已发起过{WorkflowModule.Modules[WorkflowModuleConsts.KnowledgeDelete]}流程");

                //调用流程创建
                var workFlowId = await _workflowApplication.StartWorkflowAsync(workDto, HttpContext.RequestAborted);

                //知识审批主表
                await _knowledgeDomainService.AddWorkFlow(know.Id, workFlowId, EKnowledgeApplyType.Delete, HttpContext.RequestAborted);

                ////修改知识对应流程
                //know.WorkflowId = workFlowId;
                //await _knowledgeRepository.UpdateAsync(know, HttpContext.RequestAborted);
            }
            else
                throw UserFriendlyException.SameMessage("知识删除失败");
        }

        /// <summary>
        /// 知识库-知识下架
        /// </summary>
        /// <param name="Id"></param>
        /// <returns></returns>
        [Permission(EPermission.KnowledgeOffShelf)]
        [HttpPut("offshelf")]
        public async Task KnowledgeOffShelf(string Id)
        {
            var know = await _knowledgeRepository.GetAsync(Id, HttpContext.RequestAborted);
            if (know != null && know.Status == EKnowledgeStatus.OnShelf)
            {
                know.Status = EKnowledgeStatus.OffShelf;
                know.OnShelfTime = null;
                know.OffShelfTime = DateTime.Now;
                await _knowledgeRepository.UpdateAsync(know, HttpContext.RequestAborted);
            }
            else
                throw UserFriendlyException.SameMessage("知识下架失败");

        }

        /// <summary>
        /// 知识库-知识上架
        /// </summary>
        /// <param name="Id"></param>
        /// <returns></returns>
        [Permission(EPermission.KnowledgeOnTheShelf)]
        [HttpPut("ontheshelf")]
        public async Task KnowledgeOnTheShelf(string Id)
        {
            var know = await _knowledgeRepository.GetAsync(Id, HttpContext.RequestAborted);
            if (know != null && know.Status == EKnowledgeStatus.OffShelf)
            {
                know.Status = EKnowledgeStatus.OnShelf;
                know.OnShelfTime = DateTime.Now;
                know.OffShelfTime = null;
                await _knowledgeRepository.UpdateAsync(know, HttpContext.RequestAborted);
            }
            else
                throw UserFriendlyException.SameMessage("知识上架失败");
        }

        /// <summary>
        /// 撤销
        /// </summary>
        /// <param name="workFlowId"></param>
        /// <returns></returns>
        [HttpPut("revoke")]
        public async Task KnowledgeRevoke(string workFlowId)
        {
            await _knowledgeDomainService.KnowledgeRevoke(workFlowId, HttpContext.RequestAborted);
        }
        #endregion

        #region 知识列表
        /// <summary>
        /// 知识查询
        /// </summary>
        /// <param name="pagedDto"></param>
        /// <returns></returns>
        [Permission(EPermission.KnowledgeAllList)]
        [HttpGet()]
        public async Task<PagedDto<KnowledgeDataDto>> GetKnowList([FromBody] KnowPagedListDto pagedDto)
        {
            return await _knowApplication.GetKnowList(pagedDto);
        }

        #endregion

        #region 知识检索
        /// <summary>
        /// 知识检索
        /// </summary>
        /// <param name="pagedDto"></param>
        /// <returns></returns>
        [Permission(EPermission.KnowledgeRetrieval)]
        [HttpGet("knowretrieval")]
        public async Task<PagedDto<KnowledgeRetrievalDataDto>> KnowRetrieval([FromBody] KnowledgeRetrievalPagedListDto pagedDto)
        {
            if (string.IsNullOrEmpty(pagedDto.Keyword))
                throw UserFriendlyException.SameMessage("请输入关键字");

            RefAsync<int> total = 0;
            var temp = await _knowledgeRepository
              .Queryable(true)
              .Where(p => p.Status == EKnowledgeStatus.OnShelf && p.IsDeleted == false)
              .WhereIF(pagedDto.RetrievalType == null || pagedDto.RetrievalType == EKnowledgeRetrievalType.All, d => d.Title.Contains(pagedDto.Keyword) || d.Content.Contains(pagedDto.Keyword) || d.Additions.Contains(pagedDto.Keyword))
              .WhereIF(pagedDto.RetrievalType == EKnowledgeRetrievalType.Title, d => d.Title.Contains(pagedDto.Keyword))
              .WhereIF(pagedDto.RetrievalType == EKnowledgeRetrievalType.Content, d => d.Content.Contains(pagedDto.Keyword))
              .WhereIF(pagedDto.RetrievalType == EKnowledgeRetrievalType.Enclosure, d => d.Additions.Contains(pagedDto.Keyword))

              .Select(o => new
              {
                  index = SqlFunc.RowNumber($"{o.Version} desc ", $"{o.Code}"),
                  o.Id,
                  o.Code,
                  o.Title,
                  o.Summary,
                  o.Content,
                  o.PageView
              })

              .OrderByDescending(p => p.PageView)
             .ToPageListAsync(pagedDto.PageIndex, pagedDto.PageSize, total);

            return new PagedDto<KnowledgeRetrievalDataDto>(total, _mapper.Map<IReadOnlyList<KnowledgeRetrievalDataDto>>(temp));
        }
        #endregion

        #region 我的知识删除列表
        /// <summary>
        /// 我的知识删除列表
        /// </summary>
        /// <param name="pagedDto"></param>
        /// <returns></returns>
        [HttpGet("deletelist")]
        public async Task<PagedDto<KnowledgeDataDto>> GetDeleteApplyList([FromBody] KnowledgeDeletelPagedListDto pagedDto)
        {
            var (total, items) = await _knowledgeWorkFlowRepository
               .Queryable(true)
               .Includes(it => it.Knowledge)
               .Includes(it => it.Knowledge.User)
               .Includes(it => it.Knowledge.SystemOrganize)
               .Includes(it => it.Knowledge.KnowledgeType)
               .Includes(it => it.Workflow)
               .WhereIF(pagedDto.EKnowledgeWorkFlowStatus.HasValue, d => d.WorkFlowApplyStatus == pagedDto.EKnowledgeWorkFlowStatus)
               .WhereIF(pagedDto.StartApplyTime.HasValue, d => d.CreationTime >= pagedDto.StartApplyTime)
               .WhereIF(pagedDto.EndApplyTime.HasValue, d => d.CreationTime <= pagedDto.EndApplyTime)
               .WhereIF(!string.IsNullOrEmpty(pagedDto.Keyword), d => d.Knowledge.User.Name.Contains(pagedDto.Keyword!)
               || d.Knowledge.SystemOrganize.OrgName.Contains(pagedDto.Keyword!)
               || d.Knowledge.Title.Contains(pagedDto.Keyword!))

               .OrderByDescending(p => p.CreationTime)
               .ToPagedListAsync(pagedDto.PageIndex, pagedDto.PageSize, HttpContext.RequestAborted);

            return new PagedDto<KnowledgeDataDto>(total, _mapper.Map<IReadOnlyList<KnowledgeDataDto>>(items));
        }

        #endregion

        #region 审核管理
        /// <summary>
        /// 审核管理
        /// </summary>
        /// <param name="pagedDto"></param>
        /// <returns></returns>
        [Permission(EPermission.KnowledgeApprovedAllList)]
        [HttpGet("approvedlist")]
        public async Task<PagedDto<KnowledgeApprovalDataDto>> ApprovedList([FromBody] KnowledgeApprovalPagedListDto pagedDto)
        {
            var (total, items) = await _knowledgeWorkFlowRepository
                .Queryable(true)
                .Includes(it => it.Knowledge)
                .Includes(it => it.Knowledge.User)
                .Includes(it => it.Knowledge.SystemOrganize)
                .Includes(it => it.Knowledge.KnowledgeType)
                .Includes(it => it.Workflow)

                .WhereIF(pagedDto.EKnowledgeApplyType.HasValue, d => d.WorkflowModuleStatus == pagedDto.EKnowledgeApplyType)
                .WhereIF(pagedDto.EKnowledgeWorkFlowStatus.HasValue, d => d.WorkFlowApplyStatus == pagedDto.EKnowledgeWorkFlowStatus)
                .WhereIF(!string.IsNullOrEmpty(pagedDto.Keyword), d => d.Knowledge.User.Name.Contains(pagedDto.Keyword!)
                || d.Knowledge.SystemOrganize.OrgName.Contains(pagedDto.Keyword!)
                || d.Knowledge.Title.Contains(pagedDto.Keyword!))

                .OrderByDescending(p => p.CreationTime)
                .ToPagedListAsync(pagedDto.PageIndex, pagedDto.PageSize, HttpContext.RequestAborted);


            return new PagedDto<KnowledgeApprovalDataDto>(total, _mapper.Map<IReadOnlyList<KnowledgeApprovalDataDto>>(items));
        }
        #endregion

    }
}
