﻿using Hotline.FlowEngine.Notifies;
using Hotline.KnowledgeBase;
using Hotline.Settings;
using Hotline.Share.Enums.Order;
using MediatR;

namespace Hotline.Application.Handlers.FlowEngine;

public class EndWorkflowHandler : INotificationHandler<EndWorkflowNotify>
{
    private readonly IKnowledgeDomainService _knowledgeDomainService;

    public EndWorkflowHandler(IKnowledgeDomainService knowledgeDomainService)
    {
        _knowledgeDomainService = knowledgeDomainService;
    }

    /// <summary>Handles a notification</summary>
    /// <param name="notification">The notification</param>
    /// <param name="cancellationToken">Cancellation token</param>
    public async Task Handle(EndWorkflowNotify notification, CancellationToken cancellationToken)
    {
        var workflow = notification.Workflow;

        switch (workflow.ModuleCode)
        {
            case WorkflowModuleConsts.KnowledgeAdd://新增知识库
            case WorkflowModuleConsts.KnowledgeUpdate://修改知识库
            case WorkflowModuleConsts.KnowledgeDelete://删除知识库
                await _knowledgeDomainService.EndWorkKnowledge(workflow, cancellationToken);
                break;
            case WorkflowModuleConsts.TelRestApply:
                //TODO 审核通过
                //notification.Workflow.ExternalId;
                break;

        }
    }
}