﻿using Hotline.Share.Enums.KnowledgeBase;

namespace Hotline.Share.Dtos.Knowledge
{
    public record KnowledgeBaseDataDto
    {
        /// <summary>
        /// 知识ID
        /// </summary>
        public string Id { get; set; }

        /// <summary>
        /// 标题
        /// </summary>
        public string Title { get; set; }

        /// <summary>
        /// 知识分类
        /// </summary>
        public string KnowledgeTypeName { get; set; }

        /// <summary>
        /// 热点
        /// </summary>
        public string HotspotName { get; set; }

        /// <summary>
        /// 申请人
        /// </summary>
        public string CreationName { get; set; }

        /// <summary>
        /// 浏览量
        /// </summary>
        public int PageView { get; set; }

        /// <summary>
        /// 文档状态
        /// </summary>
        public EKnowledgeStatus Status { get; set; }

        /// <summary>
        /// 流程ID
        /// </summary>
        public string WorkflowId { get; set; }
    }

    /// <summary>
    /// 知识库
    /// </summary>
    public record KnowledgeDataDto : KnowledgeBaseDataDto
    {
        /// <summary>
        /// 申请部门
        /// </summary>
        public string CreationBMName { get; set; }

        /// <summary>
        /// 是否上架
        /// </summary>
        public bool IsOnShelf { get; set; }

        /// <summary>
        /// 上架时间
        /// </summary>
        public DateTime? OnShelfTime { get; set; }

        /// <summary>
        /// 更新时间
        /// </summary>
        public DateTime? LastModificationTime { get; set; }

        /// <summary>
        /// 下架时间
        /// </summary>
        public DateTime? OffShelfTime { get; set; }
    }

    /// <summary>
    /// 审核管理
    /// </summary>
    public record KnowledgeApprovalDataDto : KnowledgeBaseDataDto
    {
        /// <summary>
        /// 申请部门
        /// </summary>
        public string CreateBMName { get; set; }

        /// <summary>
        /// 审核类型
        /// </summary>
        public string AuditType { get; set; }

        /// <summary>
        /// 时间，未审核的是到达时间，审核完成的是审核时间
        /// </summary>
        public DateTime? Time { get; set; }
    }

    /// <summary>
    /// 知识检索
    /// </summary>
    public record KnowledgeRetrievalDataDto
    {
        /// <summary>
        /// 知识ID
        /// </summary>
        public string Id { get; set; }

        /// <summary>
        /// 标题
        /// </summary>
        public string Title { get; set; }

        /// <summary>
        /// 摘要
        /// </summary>
        public string Summary { get; set; }

        /// <summary>
        /// 内容
        /// </summary>
        public string Content { get; set; }
    }
}
