﻿using XF.Domain.Dependency;

namespace Hotline.Settings;

public class SystemDomainService : ISystemDomainService, IScopeDependency
{
    private readonly ISystemOrganizeRepository _organizeRepository;

    public SystemDomainService(ISystemOrganizeRepository organizeRepository)
    {
        _organizeRepository = organizeRepository;
    }

    public async Task<IEnumerable<KeyValuePair<int, string>>> QueryOrgLevelOptionsAsync(CancellationToken cancellationToken)
    {
        var max = await _organizeRepository.Queryable()
            .MaxAsync(d => d.OrgLevel);

        var rsp = new List<KeyValuePair<int, string>>();
        for (int i = 1; i <= max; i++)
        {
            rsp.Add(new KeyValuePair<int, string>(i, $"{i}级部门"));
        }

        return rsp;
    }

    public async Task<IEnumerable<KeyValuePair<string, string>>> QueryOrgLevelStringOptionsAsync(CancellationToken cancellationToken)
    {
        var rsp = await QueryOrgLevelOptionsAsync(cancellationToken);
        return rsp.Select(d => new KeyValuePair<string, string>(d.Key.ToString(), d.Value));
    }
}